window.Gleam.initSearch([{"doc":"gleamrpc","title":"gleamrpc","content":"# gleamrpc\n\n[![Package Version](https://img.shields.io/hexpm/v/gleamrpc)](https://hex.pm/packages/gleamrpc)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/gleamrpc/)\n\nRemote Procedure Calls in Gleam !\n\nNote that this library isn&#39;t a standalone library, you will need a client and a server implementations.  \nSee gleamrpc_http_client and gleamrpc_http_server for HTTP implementations.\n\n```sh\ngleam add gleamrpc@1\n```\n```gleam\nimport convert\nimport gleam/option\nimport gleamrpc\n\npub fn main() {\n  let id_query_convert = convert.object({\n\t  use id &lt;- convert.field(&quot;id&quot;, fn(v) { Ok(v.id) }, convert.string())\n    convert.success(IdQuery(id))\n  })\n  |&gt; convert.map(\n    fn (v: IdQuery) { v.id },\n    fn (v: String) { Ok(IdQuery(v)) },\n    &quot;&quot;\n  )\n  let user_convert = convert.object({\n    use id &lt;- convert.field(&quot;id&quot;, fn(v) { Ok(v.id) }, convert.string())\n    use name &lt;- convert.field(&quot;name&quot;, fn(v) { Ok(v.name) }, convert.string())\n    use age &lt;- convert.field(&quot;age&quot;, fn(v) { Ok(v.age) }, convert.int())\n    convert.success(User(id, name, age))\n  })\n\n  let users_router = gleamrpc.Router(&quot;users&quot;, option.None) // /api/gleamRPC/users\n  let get_user = gleamrpc.query(&quot;get_user&quot;, option.Some(users_router))\n    |&gt; gleamrpc.params(id_query_convert) // /api/gleamRPC/users/get_user?id=&lt;ID&gt;\n    |&gt; gleamrpc.returns(user_convert)\n\n  // user is of type Result(User, gleamrpc.GleamRPCError)\n  let user = get_user\n  |&gt; gleamrpc.with_client(client) // client is provided by the implementation you choose\n  |&gt; gleamrpc.call(&quot;1&quot;)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/gleamrpc&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"gleamrpc","title":"GleamRPCError","content":"pub type GleamRPCError(error) {\n  GleamRPCError(error: error)\n}\n\nGleamRPCError(error: error)\n\n\nSynonyms:\ngleamrpc.GleamRPCError\ngleamrpc GleamRPCError","url":"gleamrpc.html#GleamRPCError"},{"doc":"gleamrpc","title":"Procedure","content":"pub type Procedure(params, return) {\n  Procedure(\n    name: String,\n    router: option.Option(Router),\n    type_: ProcedureType,\n    params_type: convert.Converter(params),\n    return_type: convert.Converter(return),\n  )\n}\n\nProcedure(\n  name: String,\n  router: option.Option(Router),\n  type_: ProcedureType,\n  params_type: convert.Converter(params),\n  return_type: convert.Converter(return),\n)\n\n\nSynonyms:\ngleamrpc.Procedure\ngleamrpc Procedure","url":"gleamrpc.html#Procedure"},{"doc":"gleamrpc","title":"ProcedureCall","content":"pub type ProcedureCall(params, return, error) {\n  ProcedureCall(\n    procedure: Procedure(params, return),\n    server: ProcedureClient(params, return, error),\n  )\n}\n\nProcedureCall(\n  procedure: Procedure(params, return),\n  server: ProcedureClient(params, return, error),\n)\n\n\nSynonyms:\ngleamrpc.ProcedureCall\ngleamrpc ProcedureCall","url":"gleamrpc.html#ProcedureCall"},{"doc":"gleamrpc","title":"ProcedureClient","content":"pub type ProcedureClient(params, return, error) {\n  ProcedureClient(\n    call: fn(Procedure(params, return), params) -&gt;\n      Result(return, GleamRPCError(error)),\n  )\n}\n\nProcedureClient(\n  call: fn(Procedure(params, return), params) -&gt;\n    Result(return, GleamRPCError(error)),\n)\n\n\nSynonyms:\ngleamrpc.ProcedureClient\ngleamrpc ProcedureClient","url":"gleamrpc.html#ProcedureClient"},{"doc":"gleamrpc","title":"ProcedureType","content":"pub type ProcedureType {\n  Query\n  Mutation\n}\n\nQuery\n\n\nMutation\n\n\nSynonyms:\ngleamrpc.ProcedureType\ngleamrpc ProcedureType","url":"gleamrpc.html#ProcedureType"},{"doc":"gleamrpc","title":"Router","content":"pub type Router {\n  Router(name: String, parent: option.Option(Router))\n}\n\nRouter(name: String, parent: option.Option(Router))\n\n\nSynonyms:\ngleamrpc.Router\ngleamrpc Router","url":"gleamrpc.html#Router"},{"doc":"gleamrpc","title":"call","content":"pub fn call(\n  procedure_call: ProcedureCall(a, b, c),\n  params: a,\n) -&gt; Result(b, GleamRPCError(c))\n\nSynonyms:\ngleamrpc.call\ngleamrpc call","url":"gleamrpc.html#call"},{"doc":"gleamrpc","title":"mutation","content":"pub fn mutation(\n  name: String,\n  router: Option(Router),\n) -&gt; Procedure(Nil, Nil)\n\nSynonyms:\ngleamrpc.mutation\ngleamrpc mutation","url":"gleamrpc.html#mutation"},{"doc":"gleamrpc","title":"params","content":"pub fn params(\n  procedure: Procedure(a, b),\n  params_converter: Converter(a),\n) -&gt; Procedure(a, b)\n\nSynonyms:\ngleamrpc.params\ngleamrpc params","url":"gleamrpc.html#params"},{"doc":"gleamrpc","title":"query","content":"pub fn query(\n  name: String,\n  router: Option(Router),\n) -&gt; Procedure(Nil, Nil)\n\nSynonyms:\ngleamrpc.query\ngleamrpc query","url":"gleamrpc.html#query"},{"doc":"gleamrpc","title":"returns","content":"pub fn returns(\n  procedure: Procedure(a, b),\n  return_converter: Converter(b),\n) -&gt; Procedure(a, b)\n\nSynonyms:\ngleamrpc.returns\ngleamrpc returns","url":"gleamrpc.html#returns"},{"doc":"gleamrpc","title":"with_client","content":"pub fn with_client(\n  procedure: Procedure(a, b),\n  server: ProcedureClient(a, b, c),\n) -&gt; ProcedureCall(a, b, c)\n\nSynonyms:\ngleamrpc.with_client\ngleamrpc with_client","url":"gleamrpc.html#with_client"},{"doc":"gleamrpc","title":"gleamrpc","content":"","url":"gleamrpc.html"}]);